@@ -57,7 +57,7 @@ env {*name*};
 {% end %}
 {% end %}
 
-{% if use_apisix_openresty then %}
+{% if use_apisix_base then %}
 thread_pool grpc-client-nginx-module threads=1;
 
 lua {
@@ -75,12 +75,26 @@ http {
                       .. [=[{*lua_cpath*};";
 
     {% if enabled_stream_plugins["prometheus"] then %}
+
+    init_by_lua_block {
+        require "resty.core"
+        local process = require("ngx.process")
+        local ok, err = process.enable_privileged_agent()
+        if not ok then
+            ngx.log(ngx.ERR, "failed to enable privileged_agent: ", err)
+        end
+    }
+
     init_worker_by_lua_block {
         require("apisix.plugins.prometheus.exporter").http_init(true)
     }
 
     server {
-        listen {* prometheus_server_addr *};
+        {% if use_apisix_base then %}
+            listen {* prometheus_server_addr *} enable_process=privileged_agent;
+        {% else %}
+            listen {* prometheus_server_addr *};
+        {% end %}
 
         access_log off;
 
@@ -201,7 +215,7 @@ stream {
 
         proxy_pass apisix_backend;
 
-        {% if use_apisix_openresty then %}
+        {% if use_apisix_base then %}
         set $upstream_sni "apisix_backend";
         proxy_ssl_server_name on;
         proxy_ssl_name $upstream_sni;
@@ -381,7 +395,7 @@ http {
     upstream apisix_backend {
         server 0.0.0.1;
 
-        {% if use_apisix_openresty then %}
+        {% if use_apisix_base then %}
         keepalive {* http.upstream.keepalive *};
         keepalive_requests {* http.upstream.keepalive_requests *};
         keepalive_timeout {* http.upstream.keepalive_timeout *};
@@ -417,7 +431,7 @@ http {
     }
     {% end %}
 
-    {% if use_apisix_openresty then %}
+    {% if use_apisix_base then %}
     apisix_delay_client_max_body_check on;
     apisix_mirror_on_demand on;
     {% end %}
@@ -469,7 +483,11 @@ http {
 
     {% if enabled_plugins["prometheus"] and prometheus_server_addr then %}
     server {
-        listen {* prometheus_server_addr *};
+        {% if use_apisix_base then %}
+            listen {* prometheus_server_addr *} enable_process=privileged_agent;
+        {% else %}
+            listen {* prometheus_server_addr *};
+        {% end %}
 
         access_log off;
 
@@ -677,12 +695,6 @@ http {
             if ($http_x_forwarded_for != "") {
                 set $var_x_forwarded_for "${http_x_forwarded_for}, ${realip_remote_addr}";
             }
-            if ($http_x_forwarded_host != "") {
-                set $var_x_forwarded_host $http_x_forwarded_host;
-            }
-            if ($http_x_forwarded_port != "") {
-                set $var_x_forwarded_port $http_x_forwarded_port;
-            }
 
             proxy_set_header   X-Forwarded-For      $var_x_forwarded_for;
             proxy_set_header   X-Forwarded-Proto    $var_x_forwarded_proto;
@@ -734,7 +746,7 @@ http {
                 apisix.grpc_access_phase()
             }
 
-            {% if use_apisix_openresty then %}
+            {% if use_apisix_base then %}
             # For servers which obey the standard, when `:authority` is missing,
             # `host` will be used instead. When used with apisix-base, we can do
             # better by setting `:authority` directly
@@ -787,7 +799,7 @@ http {
         location = /proxy_mirror {
             internal;
 
-            {% if not use_apisix_openresty then %}
+            {% if not use_apisix_base then %}
             if ($upstream_mirror_uri = "") {
                 return 200;
             }
