@@ -61,7 +61,12 @@ local etcd_schema = {
             default = 30,
             minimum = 1,
             description = "etcd connection timeout in seconds",
-        }
+        },
+        use_grpc = {
+            type = "boolean",
+            -- TODO: set true by default in v3.2
+            default = false,
+        },
     },
     required = {"prefix", "host"}
 }
